@@ -40,7 +40,7 @@ aicVC <-
    if(!all(is.finite(y)))
       stop("y: non-numeric or infinite data points not allowed.")
    if(!missing(x))
-      if(any(is.infinite(x) | is.na(x)))
+      if(any(sapply(x,is.infinite) | sapply(x,is.na)))
          stop("x: missing or infinite data points not allowed.")
    UseMethod("aicVC")
 }
@@ -965,7 +965,7 @@ mAIC<-
    if(!all(is.finite(y)))
       stop("y: non-numeric or infinite data points not allowed.")
    if(!missing(x))
-      if(any(is.infinite(x) | is.na(x)))
+      if(any(sapply(x,is.infinite) | sapply(x,is.na)))
          stop("x: missing or infinite data points not allowed.")
    if(is.null(prdat) || !is.element("Pr",class(prdat))){
       mAIC.default(y = y,
